"""Version information."""
__title__ = 'celery-slack-webhooks'
__description__ = 'A Slack extension for Celery.'
__url__ = 'https://github.com/leigh-johnson/celery-slack'
__version__ = '0.4.5-rc2'
__author__ = 'Leigh Johnson'
__author_email__ = 'ljohnson@slack-corp.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2017-2018 Christopher Flynn, 2019 Leigh Johnson'
__docs_copyright__ = '2017-2018 Christopher Flynn, 2019 Leigh Johnson'
